# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'net/https'
require 'uri'
require 'ostruct'
require 'json'

# rubocop:disable Style/Documentation
module Puppet_X
  module EnterpriseModules
    module Zfs
      class Connection
        # rubocop:enable Style/Documentation
        DEFAULT_ENDPOINT = 'https://specify.your.host.com/'
        DEFAULT_USERNAME = 'root'

        VERB_MAP = {
          :get => Net::HTTP::Get,
          :post => Net::HTTP::Post,
          :put => Net::HTTP::Put,
          :delete => Net::HTTP::Delete
        }.freeze

        VALID_OPTIONS = [:endpoint, :username, :password, :verify_ssl].freeze
        # rubocop:disable Metrics/PerceivedComplexity
        # rubocop:disable Metrics/AbcSize
        # rubocop:disable Metrics/CyclomaticComplexity
        def initialize(options)
          # rubocop:enable Metrics/PerceivedComplexity
          # rubocop:enable Metrics/AbcSize
          # rubocop:enable Metrics/CyclomaticComplexity
          fail "options #{options} contains invalid key" unless options.keys.all? { |key| valid_option?(key) }

          endpoint = options.fetch(:endpoint) { DEFAULT_ENDPOINT }
          @username   = options.fetch(:username) { DEFAULT_USERNAME }
          @password   = options.fetch(:password) { fail 'You must pass a password' }
          # rubocop:disable Lint/BooleanSymbol
          # rubocop:disable Style/RedundantFetchBlock
          verify_ssl = options.fetch(:verify_ssl) { :true } == :true
          # rubocop:enable Lint/BooleanSymbol
          # rubocop:enable Style/RedundantFetchBlock

          uri = URI.parse(endpoint)
          @header = {}
          @http = Net::HTTP.new(uri.host, uri.port)
          # rubocop:disable Style/GuardClause
          if uri.scheme == 'https'
            # rubocop:enable Style/GuardClause
            @http.use_ssl = true
            @http.verify_mode = OpenSSL::SSL::VERIFY_NONE unless verify_ssl
          end
        end

        def get(path, params = {})
          request_json :get, path, params
        end

        def post(path, params)
          request_json :post, path, params
        end

        def put(path, params)
          request_json :put, path, params
        end

        def delete(path)
          request_json :delete, path, {}
        end

        private

        def request_json(method, path, params)
          response = request(method, path, params)
          body = response.body ? JSON.parse(response.body) : {}
          fail "Error on #{method} to #{path}\nstatus #{response.code}, message: #{body['fault']['message']}" if error?(response)

          # rubocop:disable Style/OpenStructUse
          OpenStruct.new(:code => response.code, :body => body)
          # rubocop:enable Style/OpenStructUse
        rescue JSON::ParserError
          response
        end

        def error?(response)
          response.code[0] != '2'
        end

        def valid_option?(option_key)
          VALID_OPTIONS.include?(option_key)
        end

        def request(method, path, params = {})
          case method
          when :get
            full_path = encode_path_params(path, params)
            request = VERB_MAP[method.to_sym].new(full_path, @header)
          else
            request = VERB_MAP[method.to_sym].new(path, @header)
            request.body = params.to_json
            request.content_type = 'application/json'
          end
          request.basic_auth(@username, @password)
          @http.request(request)
        end

        def encode_path_params(path, params)
          encoded = URI.encode_www_form(params)
          [path, encoded].join('?')
        end
      end
    end
  end
end
