# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
# rubocop:disable Style/Documentation
module Puppet_X
  module EnterpriseModules
    module Zfs
      module Type
        # rubocop:enable Style/Documentation
        Puppet_X::EnterpriseModules::Settings::SETTINGS_FILE = '/etc/zfs_appliance.yaml'
        PUPPET_ATTRIBUTES = [:name, :alias, :audit, :before, :loglevel,
                             :noop, :notify, :require, :schedule, :stage,
                             :subscribe, :tag, :provider, :pool_name,
                             :share_name, :project_name, :appliance,
                             :maxblocksize, :ensure, :casesensitivity].freeze

        # @private
        def self.included(parent)
          #
          # We use send here because in ruby 1.8.7, include is a private
          # method. In ruby 1.9 this has become public.
          # TODO: When we lose support for ruby 1.8.7 change this to more standard
          #
          parent.send(:include, EasyType)
          parent.send(:include, Settings)
          parent.extend(ClassMethods)
          parent.extend(Puppet_X::EnterpriseModules::Zfs::TitleParser)
        end

        def on_create(_command_builder)
          Puppet.info "create #{name}"
          with_appliance_connection do |connection|
            path = restfull_path.subst(to_hash)
            connection.post(path, normalized_data)
          end
          nil
        end

        def on_modify(_command_builder)
          Puppet.info "modify #{name}"
          with_appliance_connection do |connection|
            path = restfull_path.subst(to_hash)
            path = "#{path}/#{key_value}"
            connection.put(path, normalized_data)
          end
          nil
        end

        def on_destroy(_command_builder)
          Puppet.info "destroy #{name}"
          with_appliance_connection do |connection|
            path = restfull_path.subst(to_hash)
            path = "#{path}/#{key_value}"
            connection.delete(path)
          end
          nil
        end

        def normalized_data
          hash = to_hash.reject { |key, _value| PUPPET_ATTRIBUTES.include?(key) }
          hash[:name] = key_value
          hash
        end

        def restfull_path(restfull_path = nil)
          self.class.restfull_path(restfull_path)
        end

        private

        def with_appliance_connection
          options = options_for(appliance)
          connection = Puppet_X::EnterpriseModules::Zfs::Connection.new(options)
          yield(connection)
        end

        def options_for(appliance)
          settings = configuration.fetch(appliance)
          {
            :endpoint => settings['appliance_url'],
            :username => settings['username'],
            :password => settings['password'],
            :verify_ssl => settings['verify_ssl']
          }
        end

        def key_name
          self.class.key_name
        end

        def key_value
          self[key_name]
        end

        # rubocop:disable Style/Documentation
        module ClassMethods
          # rubocop:enable Style/Documentation
          # This is a temporary fix. to use a private shared folder for the properties.
          # This to make sure no name clashes occur. On a new version of easy_type, we can
          # get rid of this fix.
          # TODO: Remove this when moved to a newer version of easy_type
          def shared_file(parameter_name)
            get_ruby_file("puppet/type/shared/zfs_appliance/#{parameter_name}")
          end

          def get_raw_resources
            configuration.keys.collect do |appliance|
              settings = configuration[appliance]
              options = {
                :endpoint => settings['appliance_url'],
                :username => settings['username'],
                :password => settings['password'],
                :verify_ssl => settings['verify_ssl']
              }
              connection = Puppet_X::EnterpriseModules::Zfs::Connection.new(options)
              resources_at(appliance, restfull_path, {}, [], connection)
            end.flatten
          end

          # rubocop:disable Metrics/AbcSize
          def resources_at(appliance, path, config, resources, connection)
            # rubocop:enable Metrics/AbcSize
            resulting_path = path.subst(config)
            if path.unassigned(resulting_path).empty?
              elements = connection.get(resulting_path).body[restfull_type]
              config['appliance'] = appliance
              resources << elements.collect { |m| [m.merge(config)] }.flatten
            else
              children = children(resulting_path, connection)
              children.each do |child|
                new_config = config.dup
                new_config[path.unassigned(resulting_path).first] = child
                resources_at(appliance, path, new_config, resources, connection)
              end
            end
            resources.flatten
          end

          def key_name
            key_attributes.first
          end

          def children(path, connection)
            data = connection.get(path)
            type = data.body.keys.first
            data.body[type].collect { |e| e['name'] }
          end

          def restfull_path(restfull_path = nil)
            if restfull_path
              @restfull_path = Puppet_X::EnterpriseModules::RestfullPath.new(restfull_path)
            else
              @restfull_path
            end
          end

          def restfull_type(restfull_type = nil)
            if restfull_type
              @restfull_type = restfull_type
            else
              @restfull_type
            end
          end
        end
      end
    end
  end
end
